'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactInnerHtml = require('react-inner-html');

var _reactInnerHtml2 = _interopRequireDefault(_reactInnerHtml);

var _stringifyObject = require('stringify-object');

var _stringifyObject2 = _interopRequireDefault(_stringifyObject);

var _sourceStyles = require('../utils/sourceStyles');

var _types = require('../utils/types');

var _helpers = require('../utils/helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Terminal = function (_React$Component) {
  (0, _inherits3.default)(Terminal, _React$Component);

  function Terminal(props) {
    (0, _classCallCheck3.default)(this, Terminal);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Terminal.__proto__ || Object.getPrototypeOf(Terminal)).call(this, props));

    _this.state = {
      commands: {},
      stdout: [],
      history: [],
      historyPosition: null,
      previousHistoryPosition: null,
      processing: false
    };

    _this.terminalRoot = _react2.default.createRef();
    _this.terminalInput = _react2.default.createRef();

    _this.focusTerminal = _this.focusTerminal.bind(_this);
    _this.validateCommands = _this.validateCommands.bind(_this);
    _this.showWelcomeMessage = _this.showWelcomeMessage.bind(_this);
    _this.showHelp = _this.showHelp.bind(_this);
    _this.pushToStdout = _this.pushToStdout.bind(_this);
    _this.getStdout = _this.getStdout.bind(_this);
    _this.clearStdout = _this.clearStdout.bind(_this);
    _this.processCommand = _this.processCommand.bind(_this);
    _this.handleInput = _this.handleInput.bind(_this);
    return _this;
  }

  (0, _createClass3.default)(Terminal, [{
    key: 'focusTerminal',
    value: function focusTerminal() {
      this.terminalInput.current.focus();
    }
  }, {
    key: 'scrollToBottom',
    value: function scrollToBottom() {
      var rootNode = this.terminalRoot.current;

      // This may look ridiculous, but it is required to decouple execution for just a millisecond in order to scroll all the way
      setTimeout(function () {
        rootNode.scrollTop = rootNode.scrollHeight;
      }, 1);
    }
  }, {
    key: 'validateCommands',
    value: function validateCommands() {
      var immutables = ['help', 'clear'];
      var commands = this.props.commands;

      var defaultCommands = {
        help: {
          description: 'Show a list of available commands.',
          fn: this.showHelp
        },
        clear: {
          description: 'Empty the terminal window.',
          explicitExec: true,
          fn: this.clearStdout
        }
      };

      var validCommands = void 0;

      // Pre-register defaults
      if (!this.props.noDefaults) validCommands = (0, _extends3.default)({}, defaultCommands);else validCommands = {};

      for (var c in commands) {
        // Check that command contains a function
        if (typeof commands[c].fn !== 'function') throw new Error('\'fn\' property of command \'' + c + '\' is invalid; expected \'function\', got \'' + (0, _typeof3.default)(commands[c].fn) + '\'');

        // Check that the command does not attempt to override immutables
        if (!this.props.noDefaults && immutables.includes(c)) throw new Error('Attempting to overwrite default command \'' + immutables[immutables.indexOf(c)] + '\'; cannot override default commands');

        // Add description if missing
        if (!commands[c].description) commands[c].description = 'None';
        validCommands[c] = commands[c];
      }

      this.setState({ commands: validCommands });
    }
  }, {
    key: 'showWelcomeMessage',
    value: function showWelcomeMessage() {
      var _this2 = this;

      var msg = this.props.welcomeMessage;

      if (typeof msg === 'boolean') this.pushToStdout('Welcome to the React terminal! Type \'help\' to get a list of commands.');else if (Array.isArray(msg)) msg.map(function (item) {
        return _this2.pushToStdout(item);
      });else this.pushToStdout(msg);
    }
  }, {
    key: 'showHelp',
    value: function showHelp() {
      for (var c in this.state.commands) {
        var cmdObj = this.state.commands[c];
        var usage = cmdObj.usage ? ' - ' + cmdObj.usage : '';
        this.pushToStdout(c + ' - ' + cmdObj.description + usage);
      }
    }
  }, {
    key: 'pushToStdout',
    value: function pushToStdout(message, rawInput) {
      var stdout = this.state.stdout;
      var history = this.state.history;

      stdout.push(message);

      if (rawInput) {
        history.push(rawInput);
        this.setState({ stdout: stdout, history: history, historyPosition: null });
      } else this.setState({ stdout: stdout });
    }
  }, {
    key: 'getStdout',
    value: function getStdout() {
      var _this3 = this;

      var lineStyle = {
        margin: '0',
        lineHeight: '21px'
      };

      return this.state.stdout.map(function (line, i) {
        var safe = _react2.default.createElement(
          'p',
          { key: i, style: lineStyle },
          line
        );
        var dangerous = _react2.default.createElement('p', (0, _extends3.default)({ key: i, style: lineStyle }, (0, _reactInnerHtml2.default)(line)));

        return _this3.props.dangerMode ? dangerous : safe;
      });
    }
  }, {
    key: 'clearStdout',
    value: function clearStdout() {
      this.setState({ stdout: [] });
    }
  }, {
    key: 'clearInput',
    value: function clearInput() {
      this.setState({ historyPosition: null });
      this.terminalInput.current.value = '';
    }
  }, {
    key: 'processCommand',
    value: function processCommand() {
      var _this4 = this;

      this.setState({ processing: true }, function () {
        var commandResult = { command: null, args: [], rawInput: null, result: null };

        var rawInput = _this4.terminalInput.current.value;

        var input = rawInput.split(' ');
        var command = input.splice(0, 1)[0]; // Removed portion is returned...
        var args = input; // ...and the rest can be used

        if (!_this4.props.noAutomaticStdout) {
          if (!_this4.props.noHistory) _this4.pushToStdout((_this4.props.promptLabel || '$') + ' ' + rawInput, rawInput);else _this4.pushToStdout((_this4.props.promptLabel || '$') + ' ' + rawInput);
        }

        if (rawInput) {
          commandResult.rawInput = rawInput;
          commandResult.command = command;
          commandResult.args = args;

          var cmdObj = _this4.state.commands[command];

          if (!cmdObj) _this4.pushToStdout(_this4.props.errorText ? _this4.props.errorText.replace(/\[command\]/gi, command) : 'Command \'' + command + '\' not found!');else {
            var res = cmdObj.fn.apply(cmdObj, (0, _toConsumableArray3.default)(args));

            _this4.pushToStdout(res);
            commandResult.result = res;
            if (cmdObj.explicitExec) cmdObj.fn.apply(cmdObj, (0, _toConsumableArray3.default)(args));
          }
        }

        _this4.setState({ processing: false }, function () {
          _this4.clearInput();
          if (!_this4.props.noAutoScroll) _this4.scrollToBottom();
          if (_this4.props.commandCallback) _this4.props.commandCallback(commandResult);
        });
      });
    }
  }, {
    key: 'scrollHistory',
    value: function scrollHistory(direction) {
      var history = (0, _helpers.cleanArray)(this.state.history).reverse(); // Clean empty items and reverse order to ease position tracking
      var position = this.state.historyPosition;
      var previousPosition = this.state.previousHistoryPosition;
      var termNode = this.terminalInput.current;

      if (!this.state.noAutomaticStdout && history.length > 0) {
        // Only run if history is non-empty and in use
        switch (direction) {
          case 'up':
            if (position === null) {
              // If at no position, get most recent entry
              termNode.value = history[0];
              this.setState({ historyPosition: 0, previousHistoryPosition: null });
            } else if (position + 1 === history.length) {
              // If the first entry will be reached on this press, get it and decrement position by 1 to avoid confusing downscroll
              termNode.value = history[history.length - 1];
              this.setState({ historyPosition: history.length - 1, previousHistoryPosition: history.length - 2 });
            } else {
              // Normal increment by one
              termNode.value = history[position + 1];
              this.setState({ historyPosition: position + 1, previousHistoryPosition: position });
            }
            break;
          case 'down':
            if (position === null || !history[position]) {
              // If at initial or out of range, clear (Unix-like behaviour)
              termNode.value = '';
              this.setState({ historyPosition: null, previousHistoryPosition: null });
            } else if (position - 1 === -1) {
              // Clear because user pressed up once and is now pressing down again => clear or is reaching bottom
              if (previousPosition === null || position === 0 && previousPosition === 1) termNode.value = '';else termNode.value = history[0];

              this.setState({ historyPosition: null, previousHistoryPosition: null });
            } else {
              // Normal decrement by one
              termNode.value = history[position - 1];
              this.setState({ historyPosition: position - 1, previousHistoryPosition: position });
            }
        }
      }
    }
  }, {
    key: 'handleInput',
    value: function handleInput(event) {
      switch (event.key) {
        case 'Enter':
          this.processCommand();break;
        case 'ArrowUp':
          this.scrollHistory('up');break;
        case 'ArrowDown':
          this.scrollHistory('down');break;
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps) {
      var oldCommands = (0, _stringifyObject2.default)(prevProps.commands);
      var currentCommands = (0, _stringifyObject2.default)(this.props.commands);

      // There was a change in commands
      if (oldCommands !== currentCommands) {
        // Re-validate
        this.validateCommands();
      }
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.validateCommands();
      if (this.props.welcomeMessage) this.showWelcomeMessage();
      if (this.props.autoFocus) this.focusTerminal();
    }
  }, {
    key: 'render',
    value: function render() {
      var styles = {
        container: (0, _extends3.default)({}, _sourceStyles.sourceStyles.container, {
          background: this.props.background || '#212121',
          backgroundSize: this.props.backgroundSize || 'cover'
        }),
        content: (0, _extends3.default)({}, _sourceStyles.sourceStyles.content, {
          color: this.props.textColor || '#FFFFFF',
          fontFamily: this.props.contentFontFamily || 'monospace'
        }),
        inputArea: (0, _extends3.default)({}, _sourceStyles.sourceStyles.inputArea),
        prompt: (0, _extends3.default)({}, _sourceStyles.sourceStyles.prompt, {
          color: this.props.promptLabelColor || '#EE9C34'
        }),
        input: (0, _extends3.default)({}, _sourceStyles.sourceStyles.input, {
          color: this.props.promptTextColor || '#F0BF81',
          fontFamily: this.props.inputFontFamily || 'monospace'
        })
      };

      return _react2.default.createElement(
        'div',
        {
          ref: this.terminalRoot,
          className: this.props.className || null,
          name: 'react-console-emulator',
          style: styles.container,
          onClick: this.focusTerminal
        },
        _react2.default.createElement(
          'div',
          {
            className: this.props.contentClassName || null,
            name: 'react-console-emulator__content',
            style: styles.content
          },
          this.getStdout(),
          _react2.default.createElement(
            'div',
            {
              className: this.props.inputAreaClassName || null,
              name: 'react-console-emulator__inputArea',
              style: styles.inputArea
            },
            _react2.default.createElement(
              'span',
              {
                className: this.props.promptLabelClassName || null,
                style: styles.prompt
              },
              this.props.promptLabel || '$'
            ),
            _react2.default.createElement('input', {
              ref: this.terminalInput,
              className: this.props.inputClassName || null,
              name: 'react-console-emulator__input',
              style: styles.input,
              type: 'text',
              onKeyDown: this.handleInput,
              disabled: this.props.disableOnProcess && this.state.processing
            })
          )
        )
      );
    }
  }], [{
    key: 'manualPushToStdout',


    /**
     * Manually push to the stdout of a terminal. Use with caution.
     * @param {String} message The message to output to the terminal. If not using safe mode, make sure to XSS-proof this.
     * @param {Boolean} dangerMode If true, set output content with innerHTML. Dangerous.
     * @param {HTMLElement} contentElement The content element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputElement The input element of the terminal you want to push output to. Uses first found element if omitted.
     * @param {HTMLElement} inputAreaElement The input area element of the terminal you want to push output to. Uses first found element if omitted.
     */
    value: function manualPushToStdout(message, dangerMode, contentElement, inputElement, inputAreaElement) {
      console.warn('DeprecationWarning: Terminal.manualPushToStdout() is deprecated and will be removed in a future release. For more information, see https://github.com/js-rcon/react-console-emulator/blob/master/LEGACY.md');

      var content = contentElement || document.getElementsByName('react-console-emulator__content')[0];
      var input = inputElement || document.getElementsByName('react-console-emulator__input')[0];
      var inputArea = inputAreaElement || document.getElementsByName('react-console-emulator__inputArea')[0];

      var messageElement = document.createElement('p');

      if (dangerMode) messageElement.innerHTML = message;else messageElement.innerText = message;

      messageElement.style = 'margin: 0px; line-height: 21px;';

      content.appendChild(messageElement);
      content.appendChild(inputArea);
      input.value = '';
      input.focus();
    }
  }]);
  return Terminal;
}(_react2.default.Component);

Terminal.propTypes = (0, _extends3.default)({}, _types.types);
exports.default = Terminal;